"""Fixed key modelset index

Peek Plugin Database Migration Script

Revision ID: ab9da4532175
Revises: b0b08aff8542
Create Date: 2017-07-16 13:38:20.517763

"""

# revision identifiers, used by Alembic.
revision = "ab9da4532175"
down_revision = "b0b08aff8542"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index(
        "idx_LiveDbDKey_modelSet_key",
        "LiveDbItem",
        ["modelSetId", "key"],
        unique=True,
        schema="pl_livedb",
    )
    op.drop_index(
        "idx_LiveDbDKey_liveDbKey", table_name="LiveDbItem", schema="pl_livedb"
    )
    op.drop_index(
        "idx_LiveDbDKey_modelSetId", table_name="LiveDbItem", schema="pl_livedb"
    )
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_index(
        "idx_LiveDbDKey_modelSetId",
        "LiveDbItem",
        ["modelSetId"],
        unique=False,
        schema="pl_livedb",
    )
    op.create_index(
        "idx_LiveDbDKey_liveDbKey",
        "LiveDbItem",
        ["key"],
        unique=False,
        schema="pl_livedb",
    )
    op.drop_index(
        "idx_LiveDbDKey_modelSet_key", table_name="LiveDbItem", schema="pl_livedb"
    )
    # ### end Alembic commands ###



"""Added unique constraing to model set

Peek Plugin Database Migration Script

Revision ID: b0b08aff8542
Revises: 64309a77b48d
Create Date: 2017-07-13 20:48:40.825705

"""

# revision identifiers, used by Alembic.
revision = "b0b08aff8542"
down_revision = "64309a77b48d"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa
import geoalchemy2


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.create_unique_constraint(None, "LiveDbModelSet", ["name"], schema="pl_livedb")
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_constraint(None, "LiveDbModelSet", schema="pl_livedb", type_="unique")
    # ### end Alembic commands ###



from peek_plugin_livedb._private.PluginNames import livedbTuplePrefix

from vortex.Tuple import Tuple, addTupleType


@addTupleType
class LiveDbRawValueUpdateTuple(Tuple):
    """Live DB Raw Value Update

    This tuple represents an update to the raw value for a live db item

    """

    __tupleType__ = livedbTuplePrefix + "LiveDbRawValueUpdateTuple"
    __slots__ = ("key", "rawValue")



"""Fixed nullable Queue rawValue

Peek Plugin Database Migration Script

Revision ID: 1f014c013f8b
Revises: d4ffb9c5cb27
Create Date: 2020-04-21 18:33:51.189917

"""

# revision identifiers, used by Alembic.
revision = "1f014c013f8b"
down_revision = "d4ffb9c5cb27"
branch_labels = None
depends_on = None

import sqlalchemy as sa
from alembic import op


def upgrade():
    schema = "pl_livedb"
    table = "LiveDbRawValueQueue"
    op.execute(
        '''CREATE TEMP TABLE tempqueue AS
                  select * from %s."%s"'''
        % (schema, table)
    )

    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_table("LiveDbRawValueQueue", schema="pl_livedb")

    op.create_table(
        "LiveDbRawValueQueue",
        sa.Column("id", sa.BigInteger(), autoincrement=True, nullable=False),
        sa.Column("modelSetId", sa.Integer(), nullable=False),
        sa.Column("key", sa.String(), nullable=False),
        sa.Column("rawValue", sa.String(), nullable=True),
        sa.PrimaryKeyConstraint("id"),
        schema="pl_livedb",
    )
    op.create_index(
        "idx_LiveDbRawValueQueue_all",
        "LiveDbRawValueQueue",
        ["id", "modelSetId", "key", "rawValue"],
        unique=False,
        schema="pl_livedb",
    )
    # ### end Alembic commands ###

    op.execute(
        '''SELECT setval('%s."%s_id_seq"', max(id))
                  FROM %s."%s"'''
        % (schema, table, schema, table)
    )

    op.execute(
        """INSERT INTO %s."%s"
                  select * from tempqueue"""
        % (schema, table)
    )

    op.execute("""DROP TABLE tempqueue""")


def downgrade():
    pass



import logging
from typing import List

from twisted.internet import defer
from twisted.internet.defer import Deferred, inlineCallbacks

from peek_plugin_livedb._private.server.LiveDBReadApi import LiveDBReadApi
from peek_plugin_livedb._private.server.controller.LiveDbController import (
    LiveDbController,
)
from peek_plugin_livedb._private.server.controller.LiveDbImportController import (
    LiveDbImportController,
)
from peek_plugin_livedb._private.server.controller.LiveDbValueUpdateQueueController import (
    LiveDbValueUpdateQueueController,
)
from peek_plugin_livedb.server.LiveDBWriteApiABC import LiveDBWriteApiABC
from peek_plugin_livedb.tuples.ImportLiveDbItemTuple import ImportLiveDbItemTuple
from peek_plugin_livedb.tuples.LiveDbRawValueUpdateTuple import (
    LiveDbRawValueUpdateTuple,
)

logger = logging.getLogger(__name__)


class LiveDBWriteApi(LiveDBWriteApiABC):
    def __init__(self):
        self._queueController = None
        self._liveDbController = None
        self._liveDbImportController = None
        self._readApi = None
        self._dbSessionCreator = None
        self._dbEngine = None

    def setup(
        self,
        queueController: LiveDbValueUpdateQueueController,
        liveDbController: LiveDbController,
        liveDbImportController: LiveDbImportController,
        readApi: LiveDBReadApi,
        dbSessionCreator,
        dbEngine,
    ):
        self._queueController = queueController
        self._liveDbController = liveDbController
        self._liveDbImportController = liveDbImportController
        self._readApi = readApi
        self._dbSessionCreator = dbSessionCreator
        self._dbEngine = dbEngine

    def shutdown(self):
        pass

    @inlineCallbacks
    def updateRawValues(
        self, modelSetName: str, updates: List[LiveDbRawValueUpdateTuple]
    ) -> Deferred:
        """Update Raw Values"""
        if not updates:
            return

        yield self._queueController.queueData(modelSetName, updates)

        self._readApi.rawValueUpdatesObservable(modelSetName).on_next(updates)

    def importLiveDbItems(
        self, modelSetName: str, newItems: List[ImportLiveDbItemTuple]
    ) -> Deferred:
        if not newItems:
            return defer.succeed(True)

        return self._liveDbImportController.importLiveDbItems(modelSetName, newItems)

    def prioritiseLiveDbValueAcquisition(
        self, modelSetName: str, liveDbKeys: List[str]
    ) -> Deferred:
        self._readApi.priorityKeysObservable(modelSetName).on_next(liveDbKeys)
        return defer.succeed(True)

    def pollLiveDbValueAcquisition(
        self, modelSetName: str, liveDbKeys: List[str]
    ) -> Deferred:
        self._readApi.pollKeysObservable(modelSetName).on_next(liveDbKeys)
        return defer.succeed(True)



import logging

from peek_plugin_base.worker.PluginWorkerEntryHookABC import PluginWorkerEntryHookABC
from peek_plugin_livedb._private.storage.DeclarativeBase import loadStorageTuples
from peek_plugin_livedb._private.tuples import loadPrivateTuples
from peek_plugin_livedb._private.worker.tasks import (
    LiveDbItemImportTask,
    LiveDbItemUpdateTask,
    BulkLoadChunkTask,
)
from peek_plugin_livedb.tuples import loadPublicTuples

logger = logging.getLogger(__name__)


class WorkerEntryHook(PluginWorkerEntryHookABC):
    def load(self):
        loadStorageTuples()
        loadPrivateTuples()
        loadPublicTuples()

        logger.debug("loaded")

    def start(self):
        logger.debug("started")

    def stop(self):
        logger.debug("stopped")

    def unload(self):
        logger.debug("unloaded")

    @property
    def celeryAppIncludes(self):
        return [
            LiveDbItemImportTask.__name__,
            LiveDbItemUpdateTask.__name__,
            BulkLoadChunkTask.__name__,
        ]



import logging
from typing import List

from sqlalchemy import select
from txcelery.defer import DeferrableTask

from peek_plugin_base.worker import CeleryDbConn
from peek_plugin_livedb._private.storage.LiveDbItem import LiveDbItem
from peek_plugin_base.worker.CeleryApp import celeryApp
from peek_plugin_livedb.tuples.LiveDbDisplayValueTuple import LiveDbDisplayValueTuple

logger = logging.getLogger(__name__)


@DeferrableTask
@celeryApp.task(bind=True)
def qryChunkInWorker(self, offset, limit) -> List[LiveDbDisplayValueTuple]:
    """Query Chunk

    This returns a chunk of LiveDB items from the database

    :param self: A celery reference to this task
    :param offset: The offset of the chunk
    :param limit: An encoded payload containing the updates
    :returns: List[LiveDbDisplayValueTuple] serialised in a payload json
    """

    table = LiveDbItem.__table__
    cols = [table.c.key, table.c.dataType, table.c.rawValue, table.c.displayValue]

    session = CeleryDbConn.getDbSession()
    try:
        result = session.execute(
            select(cols).order_by(table.c.id).offset(offset).limit(limit)
        )

        return [
            LiveDbDisplayValueTuple(
                key=o.key,
                dataType=o.dataType,
                rawValue=o.rawValue,
                displayValue=o.displayValue,
            )
            for o in result.fetchall()
        ]

    finally:
        session.close()



from vortex.Tuple import addTupleType, TupleField, Tuple

from peek_plugin_livedb._private.PluginNames import livedbTuplePrefix


@addTupleType
class AdminStatusTuple(Tuple):
    __tupleType__ = livedbTuplePrefix + "AdminStatusTuple"

    rawValueQueueStatus: bool = TupleField(False)
    rawValueQueueSize: int = TupleField(0)
    rawValueProcessedTotal: int = TupleField(0)
    rawValueLastError: str = TupleField()



from peek_plugin_livedb._private.server.LiveDBReadApi import LiveDBReadApi
from peek_plugin_livedb._private.server.LiveDBWriteApi import LiveDBWriteApi
from peek_plugin_livedb._private.server.controller.LiveDbController import (
    LiveDbController,
)
from peek_plugin_livedb._private.server.controller.LiveDbImportController import (
    LiveDbImportController,
)
from peek_plugin_livedb._private.server.controller.LiveDbValueUpdateQueueController import (
    LiveDbValueUpdateQueueController,
)
from peek_plugin_livedb.server.LiveDBApiABC import LiveDBApiABC
from peek_plugin_livedb.server.LiveDBReadApiABC import LiveDBReadApiABC
from peek_plugin_livedb.server.LiveDBWriteApiABC import LiveDBWriteApiABC


class LiveDBApi(LiveDBApiABC):
    def __init__(self):
        self._readApi = LiveDBReadApi()
        self._writeApi = LiveDBWriteApi()

    def setup(
        self,
        queueController: LiveDbValueUpdateQueueController,
        liveDbController: LiveDbController,
        liveDbImportController: LiveDbImportController,
        dbSessionCreator,
        dbEngine,
    ):
        self._readApi.setup(
            liveDbController=liveDbController,
            dbSessionCreator=dbSessionCreator,
            dbEngine=dbEngine,
        )

        self._writeApi.setup(
            queueController=queueController,
            liveDbController=liveDbController,
            liveDbImportController=liveDbImportController,
            readApi=self._readApi,
            dbSessionCreator=dbSessionCreator,
            dbEngine=dbEngine,
        )

    def shutdown(self):
        self._readApi.shutdown()
        self._writeApi.shutdown()

        self._readApi = None
        self._writeApi = None

    @property
    def writeApi(self) -> LiveDBWriteApiABC:
        return self._writeApi

    @property
    def readApi(self) -> LiveDBReadApiABC:
        return self._readApi
